class Record{
  int bibNumber;
  int itemNumber;
  String title;
  Date checkOut;
  int duration;
  
  public Record(int bibNumber, int itemNumber, String title, String checkOut, int duration){
      this.bibNumber = bibNumber;
      this.itemNumber = itemNumber;
      this.title = title;
      this.checkOut = dateParser(checkOut);
      this.duration = duration;
  }
  
  private Date dateParser(String rawInput){
      if(rawInput.equals("NULL") == true) return null;
      String[] input = split(rawInput, ' ');
      //println(input[0]);
      String[] date; 
      if(input[0].contains("-")){
        date = split(input[0], '-');
        //println(date[0] + " " + date[1] + " " + date[2]);
        return new Date(date[0], date[1], date[2]);
      }else if(input[0].contains("/")){
        date = split(input[0], '/');
        //println("20" + date[2], date[0], date[1]);
        return new Date("20" + date[2], date[0], date[1]);
      }else{
        println("New regex show up: " + input[0]);
        return null;
      }
  }
}



class DataTable{
  String tableName;
  int publishYear;
  boolean isFiction;
  ArrayList<Record> cvsTable;
  ArrayList<Integer> x;
  ArrayList<Integer> yLength;
  ArrayList<Integer> z;
  int maxDuration;
  int maxZCounter;
  
  public DataTable(String tableName, int publishYear, boolean isFiction){
      //initialization
      this.tableName = tableName;
      this.publishYear = publishYear;
      this.isFiction = isFiction;
      cvsTable = new ArrayList<Record>();
      x = new ArrayList<Integer>();
      yLength = new ArrayList<Integer>();
      z = new ArrayList<Integer>();
      //load date to create object
      loadData();
      getMaxDurationInRecord();
      setXY();
      setZ();
  }
  
  void loadData(){
      //load ArrayList<Record> cvsTable
      println("Start load data of table " + this.tableName);
      Table thisTable = loadTable(this.tableName, "header");
      for(int i = 0; i<thisTable.getRowCount(); i++){
          int bibNumber = thisTable.getInt(i, 0);
          int itemNumber = thisTable.getInt(i, 1);
          String title = thisTable.getString(i, 2);
          String checkOut = thisTable.getString(i, 3);
          int duration = thisTable.getInt(i, 4);
          this.cvsTable.add(new Record(bibNumber, itemNumber, title, checkOut, duration));
      }
      println("Table size: " + cvsTable.size());
      thisTable.clearRows();
  }
  
  void getMaxDurationInRecord() {
      int max = 0;
      for (int i=0; i<this.cvsTable.size(); i++) {
          int cDuration = this.cvsTable.get(i).duration;
          if (cDuration > max) {
            max = cDuration;
          }
      }
      this.maxDuration = max;
      println("MaxDuration of " + this.tableName + " is: " + this.maxDuration);
  }  
  
  void setZ(){
      int max = 0;
      int zCounter = 0;
      Date today = this.cvsTable.get(0).checkOut;
      Date yesterday = new Date(today.year, today.month, today.day - 1);
      for (int i=0; i<this.cvsTable.size(); i++) {
          today = this.cvsTable.get(i).checkOut;
          if(yesterday.year == today.year && yesterday.month == today.month 
             && yesterday.day == today.day){
             zCounter++;
             if(max < zCounter) max = zCounter;
          } else {
             zCounter = 0;
          }
          this.z.add(zCounter);
          yesterday = today;
      }
      this.maxZCounter = max;
      println("MaxZ of " + this.tableName + " is: " + this.maxZCounter);
      
      //map to correct z value
      for(int i = 0; i < this.z.size(); i++){
          int getZ = z.get(i);
          if(getZ != z.set(i, (int)map(getZ, 0, maxZCounter, 0, -zrange))){
            println("rewrite in wrong position while calculate z");
          }
      }
      println("Successfully calculate Z with size: " + this.z.size());
  }
  
  void setXY(){
      for(int i = 0; i < this.cvsTable.size(); i++){
          Record thisRecord = cvsTable.get(i);
          //float x = dateToday(thisRecord.checkOut);
          int cx = (int)map(dateToday(thisRecord.checkOut), 0, 365, 0, xrange);
          int cyLength = (int)map(thisRecord.duration, 0, maxDuration, 0, (yrange-100)/2);
          this.x.add(cx);
          this.yLength.add(cyLength);
      }
      println("Successfully calculate X and Y with size: "+ x.size() + " and "+ yLength.size());   
  }
  
  int[][] findBooksByName(String name){
    HashMap<Integer, ArrayList<PVector>> curves = new HashMap<Integer, ArrayList<PVector>>();
    int counter = 0;
    for(int i = 0; i < cvsTable.size(); i++){
      String newTitle = cvsTable.get(i).title;
      if(newTitle.equals(name)){
        int newItem = cvsTable.get(i).itemNumber;
        PVector newP = new PVector(x.get(i), yLength.get(i), z.get(i));
        if(curves.containsKey(newItem)){
          curves.get(newItem).add(newP);
        }else{
          //make a new ArrayList<PVector>
          ArrayList<PVector> newCurve = new ArrayList<PVector>();
          //put this vector in as first element
          newCurve.add(newP);
          //put arrayList into hashmap
          curves.put(newItem, newCurve);
        }
        counter++;
      }      
    }
    if(curves.isEmpty()){
        copies = 0;
        checkOutTimes = 0;
        bookDescription = "Sorry, no such book found...";
        return null; 
    }else{
      int[][] curvesC = new int[counter][3];
      int count = 0;
      for (Integer itemNum : curves.keySet()) {
        System.out.println("itemNumber = " + itemNum);
        ArrayList<PVector> thisList = curves.get(itemNum);
        for(int i = 0; i< thisList.size(); i++){
          PVector thisP = thisList.get(i);
          curvesC[count][0] = (int)thisP.x;
          curvesC[count][1] = (int)thisP.y;
          curvesC[count][2] = (int)thisP.z;
          count++;
        }
      }
      if(count != counter) println("Errors!!!!!!!!!!");
      
      copies = curves.size();
      checkOutTimes = counter;
      
      bookDescription = "Book:  " + name + "\nPublished in:  " + publishYear 
                        + "\nCoppies:  " + copies
                        + "\nTotal Check Out Times:  " + checkOutTimes;
      
      //remove data
      curves.clear();
      return curvesC;
    }
  }
}



class Date{
  int year;
  int month;
  int day;
  
  public Date(int year, int  month, int day){
      this.year = year;
      this.month = month;
      this.day = day;
  }
  
  public Date(String year, String  month, String day){
      this.year = Integer.parseInt(year);
      this.month = Integer.parseInt(month);
      this.day = Integer.parseInt(day);
  }
}